import unittest
from unittest.mock import patch, MagicMock
from datetime import datetime
from discogs_client.tests import DiscogsClientTestCase
from discogs_client import utils
from discogs_client.exceptions import TooManyAttemptsError
from datetime import datetime, timezone, timedelta
from discogs_client.tests import DiscogsClientTestCase
from discogs_client import utils
from dateutil.tz import tzutc, tzoffset


class UtilsTestCase(DiscogsClientTestCase):
    def test_update_qs(self):
        """update_qs helper works as intended"""
        u = utils.update_qs
        self.assertEqual(u('http://example.com', {'foo': 'bar'}), 'http://example.com?foo=bar')
        self.assertEqual(u('http://example.com?foo=bar', {'foo': 'baz'}), 'http://example.com?foo=bar&foo=baz')
        # be careful for dict iteration order is not deterministic
        result = u('http://example.com?c=3&a=yep', {'a': 1, 'b': '1'})
        try:
            self.assertEqual(result, 'http://example.com?c=3&a=yep&a=1&b=1')
        except AssertionError:
            self.assertEqual(result, 'http://example.com?c=3&a=yep&b=1&a=1')

        self.assertEqual(u('http://example.com', {'a': 't\xe9st'}),
                         'http://example.com?a=t%C3%A9st')

    def test_omit_none(self):
        o = utils.omit_none
        self.assertEqual(o({
            'foo': None,
            'baz': 'bat',
            'qux': None,
            'flan': 0,
        }), {
            'baz': 'bat',
            'flan': 0,
        })

        self.assertEqual(o({k: None for k in ('qux', 'quux', 'quuux')}), {})
        self.assertEqual(o({'nope': 'yep'}), {'nope': 'yep'})
        self.assertEqual(o({}), {})

    def test_parse_timestamp(self):
        p = utils.parse_timestamp
        self.assertEqual(
            p('2016-07-27T08:11:29-07:00'),
            datetime(2016, 7, 27, 8, 11, 29, tzinfo=tzoffset(None, -25200))
        )
        self.assertEqual(
            p('2055-07-27T08:11:29-07:00'),
            datetime(2055, 7, 27, 8, 11, 29, tzinfo=timezone(timedelta(hours=-7)))
        )
        self.assertEqual(
            p('1930-07-27T08:11:29-00:00'),
            datetime(1930, 7, 27, 8, 11, 29, tzinfo=timezone.utc)
        )
        self.assertEqual(
            p('1930-07-27T08:11:29-00:00'),
            datetime(1930, 7, 27, 8, 11, 29, tzinfo=tzutc())
        )

    def test_condition(self):
        self.assertRaises(TypeError, lambda: utils.Condition())
        self.assertEqual(utils.Condition.MINT, 'Mint (M)')
        self.assertEqual(utils.Condition.NEAR_MINT, 'Near Mint (NM or M-)')

    def test_status(self):
        self.assertRaises(TypeError, lambda: utils.Status())
        self.assertEqual(utils.Status.DRAFT, 'Draft')
        self.assertEqual(utils.Status.FOR_SALE, 'For Sale')

    def test_sort(self):
        self.assertRaises(TypeError, lambda: utils.Sort())
        self.assertEqual(utils.Sort.By.ARTIST, 'artist')
        self.assertEqual(utils.Sort.Order.ASCENDING, 'asc')
        self.assertEqual(utils.Sort.Order.DESCENDING, 'desc')


    @patch('discogs_client.utils.get_backoff_duration')
    def test_backed_off_when_rate_limit_reached(self, patched_duration):
        # Mock sleep duration returned so it doesn't effect tests speed
        patched_duration.return_value = 0

        backoff = utils.backoff

        mock_ratelimited_response = MagicMock()
        mock_ratelimited_response.status_code = 429

        mock_ok_response = MagicMock()
        mock_ok_response.status_code = 200

        call_count = 0

        class BackoffTestClass:
            def __init__(self):
                self.backoff_enabled = True

            @backoff
            def always_fails(self):
                return mock_ratelimited_response

            @backoff
            def returns_non_ratelimit_status_code(self):
                return mock_ok_response

            @backoff
            def succeeds_after_x_calls(self):
                nonlocal call_count
                call_count += 1
                if call_count < 3:
                    return mock_ratelimited_response
                else:
                    return mock_ok_response

            @backoff
            def function_not_decorated_when_disabled(self):
                return mock_ratelimited_response

        test_class = BackoffTestClass()

        with self.assertRaises(TooManyAttemptsError):
            test_class.always_fails()

        self.assertEqual(mock_ok_response, test_class.returns_non_ratelimit_status_code())
        self.assertEqual(mock_ok_response, test_class.succeeds_after_x_calls())

        with patch('discogs_client.utils.get_backoff_duration') as patched_duration:
            test_class.backoff_enabled = False
            test_class.function_not_decorated_when_disabled()
            self.assertEqual(0, patched_duration.call_count)


def suite():
    suite = unittest.TestSuite()
    suite = unittest.TestLoader().loadTestsFromTestCase(UtilsTestCase)
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



from discogs_client.fetchers import OAuth2Fetcher
import unittest
from discogs_client.tests import DiscogsClientTestCase
from discogs_client.exceptions import HTTPError


class FetcherTestCase(DiscogsClientTestCase):
    def test_memory_fetcher(self):
        """Client can fetch responses with MemoryFetcher"""
        self.m.artist(1)

        self.assertRaises(HTTPError, lambda: self.m._get('/500'))

        try:
            self.m._get('/500')
        except HTTPError as e:
            self.assertEqual(e.status_code, 500)

        self.assertRaises(HTTPError, lambda: self.m.release(1).title)
        self.assertTrue(self.m._get('/204') is None)

    def test_oauth2_fetcher(self):
        _fetcher = OAuth2Fetcher(
            'consumer_key', 'consumer_secret', token=None, secret=None)

        self.assertEqual(_fetcher.client.resource_owner_key, None)
        self.assertEqual(_fetcher.client.resource_owner_secret, None)

        query_string = b'oauth_token=token&oauth_token_secret=secret'
        token, secret = _fetcher.store_token_from_qs(query_string)
        self.assertEqual(token, 'token')
        self.assertEqual(secret, 'secret')

        _fetcher.forget_token()
        self.assertEqual(_fetcher.client.resource_owner_key, None)
        self.assertEqual(_fetcher.client.resource_owner_secret, None)

        _fetcher.store_token(token, secret)
        self.assertEqual(_fetcher.client.resource_owner_key, token)
        self.assertEqual(_fetcher.client.resource_owner_secret, secret)

        _fetcher.set_verifier('1234567890')
        self.assertEqual(_fetcher.client.verifier, '1234567890')


def suite():
    suite = unittest.TestSuite()
    suite = unittest.TestLoader().loadTestsFromTestCase(FetcherTestCase)
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import unittest
from discogs_client.models import Artist, Release, ListItem, CollectionValue
from discogs_client.tests import DiscogsClientTestCase
from discogs_client.exceptions import HTTPError


class ModelsTestCase(DiscogsClientTestCase):
    def test_artist(self):
        """Artists can be fetched and parsed"""
        a = self.d.artist(1)
        self.assertEqual(a.name, 'Persuader, The')

    def test_same_artist(self):
        """Artists can be fetched and parsed multiple times"""
        a = self.d.artist(1)
        self.assertEqual(a.name, 'Persuader, The')
        b = self.d.artist(1)
        self.assertEqual(b.name, 'Persuader, The')

    def test_release(self):
        """Releases can be fetched and parsed"""
        r = self.d.release(1)
        self.assertEqual(r.title, 'Stockholm')

    def test_master(self):
        """Masters can be fetched and parsed"""
        m = self.d.master(4242)
        self.assertEqual(len(m.tracklist), 4)

    def test_user(self):
        """Users can be fetched and parsed"""
        u = self.d.user('example')
        self.assertEqual(u.username, 'example')
        self.assertEqual(u.name, 'Example Sampleman')

    def test_list(self):
        """Lists can be fetched and parsed"""
        l = self.d.list(1)
        i = l.items
        self.assertEqual(l.name, 'Example List')
        self.assertEqual(l.description, 'description')
        self.assertEqual(l.public, True)
        self.assertTrue(isinstance(i[0], ListItem))

    def test_search(self):
        results = self.d.search('trash80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_bytes_search(self):
        results = self.d.search(b'trash80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_multiterm_search(self):
        results = self.d.search('trash', '80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_multiterm_bytes_search(self):
        results = self.d.search(b'trash', b'80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_multiterm_mixed_search(self):
        results = self.d.search('trash', b'80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_kwargs_search(self):
        results = self.d.search(artist='trash80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_kwargs_multiterm_search(self):
        results = self.d.search(title='icarus', artist='trash80')
        self.assertEqual(len(results), 13)
        self.assertTrue(isinstance(results[0], Artist))
        self.assertTrue(isinstance(results[1], Release))

    def test_utf8_search(self):
        uni_string = 'caf\xe9'
        try:
            results = self.d.search(uni_string)
        except Exception as e:
            self.fail('exception {} was raised'.format(e))

    def test_fee(self):
        fee = self.d.fee_for(20.5, currency='EUR')
        self.assertEqual(fee.currency, 'USD')
        self.assertAlmostEqual(fee.value, 1.57)

    def test_invalid_artist(self):
        """Invalid artist raises HTTPError"""
        self.assertRaises(HTTPError, lambda: self.d.artist(0).name)

    def test_invalid_release(self):
        """Invalid release raises HTTPError"""
        self.assertRaises(HTTPError, lambda: self.d.release(0).title)

    def test_http_error(self):
        """HTTPError provides useful information"""
        self.assertRaises(HTTPError, lambda: self.d.artist(0).name)

        try:
            self.d.artist(0).name
        except HTTPError as e:
            self.assertEqual(e.status_code, 404)
            self.assertEqual('404: Resource not found.', str(e))

    def test_parent_label(self):
        """Test parent_label / sublabels relationship"""
        l = self.d.label(1)
        l2 = self.d.label(31405)

        self.assertTrue(l.parent_label is None)
        self.assertTrue(l2 in l.sublabels)
        self.assertEqual(l2.parent_label, l)

    def test_master_versions(self):
        """Test main_release / versions relationship"""
        m = self.d.master(4242)
        r = self.d.release(79)
        v = m.versions

        self.assertEqual(len(v), 2)
        self.assertTrue(r in v)
        self.assertEqual(r.master, m)

        r2 = self.d.release(3329867)
        self.assertTrue(r2.master is None)

    def test_user_writable(self):
        """User profile can be updated"""
        u = self.d.user('example')
        u.name  # Trigger a fetch

        method, url, data, headers = self.d._fetcher.requests[0]
        self.assertEqual(method, 'GET')
        self.assertEqual(url, '/users/example')

        new_home_page = 'http://www.discogs.com'
        u.home_page = new_home_page
        self.assertTrue('home_page' in u.changes)
        self.assertFalse('profile' in u.changes)

        u.save()

        # Save
        method, url, data, headers = self.d._fetcher.requests[1]
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/users/example')
        self.assertEqual(data, {'home_page': new_home_page})

        # Refresh
        method, url, data, headers = self.d._fetcher.requests[2]
        self.assertEqual(method, 'GET')
        self.assertEqual(url, '/users/example')

    def test_wantlist(self):
        """Wantlists can be manipulated"""
        # Fetch the user/wantlist from the filesystem
        u = self.d.user('example')
        self.assertEqual(len(u.wantlist), 3)

        # Stub out expected responses
        self.m._fetcher.fetcher.responses = {
            '/users/example/wants/5': (b'{"id": 5}', 201),
            '/users/example/wants/1': (b'', 204),
        }

        # Now bind the user to the memory client
        u.client = self.m

        u.wantlist.add(5)
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'PUT')
        self.assertEqual(url, '/users/example/wants/5')

        u.wantlist.remove(1)
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'DELETE')
        self.assertEqual(url, '/users/example/wants/1')

    def test_inventory(self):
        """Inventory can be manipulated"""
        # Fetch users inventory from the filesystem
        u = self.d.user('example')
        self.assertEqual(len(u.inventory), 1)

        # Stub out expected responses
        self.m._fetcher.fetcher.responses = {
            '/marketplace/listings': (b'{"listing_id": 321}', 201),
        }

        # Now bind the user to the memory client
        u.client = self.m

        # Test adding by release id
        u.inventory.add_listing(release=123, condition='Mint (M)', price=15.99, status='Draft')
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/marketplace/listings')
        self.assertEqual(data['release_id'], '123')

        # test adding by release object
        r = self.d.release(1)
        self.assertEqual(r.title, 'Stockholm')
        u.inventory.add_listing(release=r, condition='Mint (M)', price=15.99, status='Draft')
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/marketplace/listings')
        self.assertEqual(data['release_id'], '1')

    def test_listing(self):
        """Listing can be manipulated"""
        # Fetch users inventory from the filesystem
        u = self.d.user('example')

        # Fetch listing
        listing = u.inventory[0]
        method, url, data, headers = self.d._fetcher.requests[1]
        self.assertEqual(method, 'GET')
        self.assertEqual(url, '/users/example/inventory?page=1&per_page=50')

        # Test fetching listing information
        self.assertEqual(listing.status, 'For Sale')
        self.assertEqual(listing.price.value, 149.99)
        self.assertEqual(listing.allow_offers, True)
        self.assertEqual(listing.id, 150899904)
        self.assertEqual(listing.seller.username, 'example')
        self.assertEqual(listing.release.id, 2992668)

        # Test manipulating listing
        listing.status = 'Draft'
        listing.price = 1.99
        # Test unsaved price
        self.assertEqual(listing.price.value, 1.99)
        listing.allow_offers = False
        expected = {
            'status': 'Draft',
            'price': 1.99,
            'allow_offers': False,
        }
        self.assertEqual(listing.changes, expected)

        # Test saving
        listing.save()
        method, url, data, headers = self.d._fetcher.requests[2]
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/marketplace/listings/150899904')
        self.assertEqual(data, expected)

        # Refresh
        method, url, data, headers = self.d._fetcher.requests[3]
        self.assertEqual(method, 'GET')
        self.assertEqual(url, '/marketplace/listings/150899904')


    def test_collection(self):
        """Collection folders can be manipulated"""
        # Fetch the users collection folders from the filesystem
        u = self.d.user('example')
        self.assertEqual(len(u.collection_folders), 3)
        # Fetch basic information from folders endpoint
        self.assertEqual(u.collection_folders[0].id, 0)
        self.assertEqual(u.collection_folders[0].name, "All")
        self.assertEqual(u.collection_folders[1].id, 1)
        self.assertEqual(u.collection_folders[1].name, "Uncategorized folder")
        # Fetch details from folders/<id>/releases endpoint
        self.assertEqual(u.collection_folders[0].releases[2].id, 656052)
        self.assertEqual(u.collection_folders[0].releases[2].release.title, "Control / Command & Conquer")

        # Mock expected responses for add_release test - FileSystemFetcher disabled now
        self.m._fetcher.fetcher.responses = {
            '/users/example/collection/folders': (b'''
                {"folders": [{"resource_url": "/users/example/collection/folders/0",
                              "id": 0,
                              "name": "All"
                             },
                             {"resource_url": "/users/example/collection/folders/1",
                              "id": 1,
                              "name": "Uncategorized folder"
                             }]
                 }''', 200),
            '/users/example/collection/folders/1': (b'{}', 200),
            '/users/example/collection/folders/1/releases/123456': (b'{"instance_id": 123}', 201),
            '/users/example/collection/folders/1/releases/1': (b'{"instance_id": 124}', 201),
        }

        # Now bind the user to the memory client
        u.client = self.m

        # test adding a release by id
        u.collection_folders[1].add_release(123456)
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/users/example/collection/folders/1/releases/123456')

        # test adding a release object
        r = self.d.release(1)
        self.assertEqual(r.title, 'Stockholm')
        u.collection_folders[1].add_release(r)
        method, url, data, headers = self.m._fetcher.last_request
        self.assertEqual(method, 'POST')
        self.assertEqual(url, '/users/example/collection/folders/1/releases/1')

    def test_delete_object(self):
        """Can request DELETE on an APIObject"""
        u = self.d.user('example')
        u.delete()

        method, url, data, headers = self.d._fetcher.last_request
        self.assertEqual(method, 'DELETE')
        self.assertEqual(url, '/users/example')

    def test_identity(self):
        """OAuth identity returns a User"""
        me = self.d.identity()
        self.assertEqual(me.data['consumer_name'], 'Test Client')
        self.assertEqual(me, self.d.user('example'))

    def test_marketplace_stats(self):
        """Release stats can be fetched and parsed"""
        stats = self.d.release(1).marketplace_stats

        # Assert that returned stats are correct.
        self.assertEqual(stats.num_for_sale, 10)
        self.assertEqual(stats.lowest_price.value, 100)
        self.assertEqual(stats.lowest_price.currency, 'EUR')

        # Assert that request URL and method is correct.
        method, url, data, headers = self.d._fetcher.requests[0]
        self.assertEqual(method, 'GET')
        self.assertEqual(url, '/marketplace/stats/1')

    def test_collection_value(self):
        """Collection Value can be fetched and parsed"""
        u = self.d.user("example")

        self.assertEqual(isinstance(u.collection_value, CollectionValue), True)
        self.assertEqual(u.collection_value.minimum, "£1.05")
        self.assertEqual(u.collection_value.maximum, "£5")
        self.assertEqual(u.collection_value.median, "£2.50")

        # Assert that request URL and method is correct.
        method, url, data, headers = self.d._fetcher.requests[0]
        self.assertEqual(method, "GET")
        self.assertEqual(url, "/users/example/collection/value")


def suite():
    suite = unittest.TestSuite()
    suite = unittest.TestLoader().loadTestsFromTestCase(ModelsTestCase)
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#!/usr/bin/env python
import os
import sys
from itertools import permutations

for name in sys.argv[1:]:
    print("doing {}".format(name))
    root, next = name.split('?')
    data, ext = next.split('.')
    elems = data.split('&')
    for permut in permutations(elems):
        link_name = "{}?{}.{}".format(root, '&'.join(permut), ext)
        if link_name == name:
            continue
        os.symlink(name, link_name)
        print("wrote {}".format(link_name))



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../../discogs_client/'))


# -- Project information -----------------------------------------------------

project = 'python3-discogs-client'
copyright = '2020-2022, The Joalla Team'
author = 'The Joalla Team'

# The full version, including alpha/beta/rc tags
release = '2.5'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'myst_parser',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

html_theme_options = {
    'collapse_navigation': False,
    'navigation_depth': 8,
    'sticky_navigation': True,
    'prev_next_buttons_location': 'bottom'
}

html_last_updated_fmt = "%b %d, %Y"

html_logo = '_static/discogs-vinyl-record-mark-50x50.png'

myst_heading_anchors = 7
myst_enable_extensions = [
    "substitution"
]
myst_substitutions = {
  "class": "I'm a **substitution**"
}

# -- autodoc tuning -------------------------------------------------
# don't show docstring of parent classes on childs
autodoc_inherit_docstrings = False

# show docstring of class AND __init__ method
autoclass_content = 'both'

autodoc_default_options = {
    'members': True,
    'undoc-members': True,
    'show-inheritance': True,
    'private-members': True,
    'member-order': 'bysource',
}



#!/usr/bin/env python

from setuptools import setup

setup(
        name='python3-discogs-client',
        version='2.5',
        description='Python API client for Discogs',
        long_description='This is an active fork of the official "Discogs API client for Python", which was deprecated by discogs.com as of June 2020. We think it is a very useful Python module and decided to continue maintaining it. Please visit: https://github.com/joalla/discogs_client for more information.',
        url='https://github.com/joalla/discogs_client',
        author='joalla',
        author_email='jt@peek-a-boo.at',
        test_suite='discogs_client.tests',
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Console',
            'License :: OSI Approved :: BSD License',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Communications',
            'Topic :: Utilities',
            ],
        install_requires=[
            'requests',
            'oauthlib',
            'python-dateutil',
            ],
        packages=[
            'discogs_client',
            ],
        )
